package com.zanxiang.sdk.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.zanxiang.common.enums.BanStatusEnum;
import com.zanxiang.common.enums.HttpStatusEnum;
import com.zanxiang.common.enums.SmsTypeEnum;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.util.pojo.ResultVO;
import com.zanxiang.mybatis.entity.GameExt;
import com.zanxiang.mybatis.entity.User;
import com.zanxiang.mybatis.entity.UserCard;
import com.zanxiang.mybatis.entity.UserShare;
import com.zanxiang.sdk.domain.dto.GameAppletDTO;
import com.zanxiang.sdk.domain.dto.UserDTO;
import com.zanxiang.sdk.domain.params.LoginMobileParam;
import com.zanxiang.sdk.domain.params.LoginPasswordParam;
import com.zanxiang.sdk.domain.params.LoginVxCodeParam;
import com.zanxiang.sdk.domain.params.UserData;
import com.zanxiang.sdk.domain.vo.UserLoginVO;
import com.zanxiang.sdk.enums.LoginTypeEnum;
import com.zanxiang.sdk.service.*;
import com.zanxiang.sdk.service.api.WxApiService;
import com.zanxiang.sdk.util.RegexUtil;
import com.zanxiang.sdk.util.RegisterUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Map;
import java.util.Objects;

/**
 * @author : lingfeng
 * @time : 2022-06-22
 * @description : 注册登录逻辑
 */
@Slf4j
@Service
public class LoginServiceImpl implements IRegisterLoginService {

    @Autowired
    private IUserTokenService userTokenService;

    @Autowired
    private IUserLoginLogService userLoginLogService;

    @Autowired
    private IUserService userService;

    @Autowired
    private ISmsService smsService;

    @Autowired
    private IWordCheckService wordCheckService;

    @Autowired
    private IPromoChannelService promoChannelService;

    @Autowired
    private IUserCardService userCardService;

    @Autowired
    private IGameUserService gameUserService;

    @Autowired
    private IGameExtService gameExtService;

    @Autowired
    private IIpBanService ipBanService;

    @Autowired
    private ICallBackService callBackService;

    @Autowired
    private IGameAppletService gameAppletService;

    @Autowired
    private WxApiService wxApiService;

    @Autowired
    private IUserShareService userShareService;

    /**
     * QQ开发者应用id
     */
    @Value("${auth.qq-appId}")
    private String qqAppId;

    /**
     * 微信开发者应用id
     */
    @Value("${auth.vx-appId}")
    private String vxAppId;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVO<UserLoginVO> loginWxCode(LoginVxCodeParam param, UserData userData) {

        log.error("小程序登录, param : {}, userData : {}", JsonUtil.toString(param), JsonUtil.toString(userData));

        //验证登录ip是否封禁
        if (ipBanService.checkIpBan(userData.getIp())) {
            return ResultVO.fail(HttpStatusEnum.IP_HALT.getMsg());
        }
        //查询微信小游戏信息
        GameAppletDTO gameAppletDTO = gameAppletService.getByGameId(userData.getGameId());
        //获取用户微信小游戏的openId
        Map<String, String> resultMap = wxApiService.getAppletOpenId(param.getCode(), gameAppletDTO.getAppId(), gameAppletDTO.getAppSecret());
        String openId = resultMap.get("openid");
        String sessionKey = resultMap.get("session_key");
        log.error("登录用户 resultMap : {}", JsonUtil.toString(resultMap));
        //根据openId查询用户
        User user = userService.getOne(new LambdaQueryWrapper<User>()
                .eq(User::getGameId, userData.getGameId()).eq(User::getOpenId, openId));
        //用户已经注册
        if (user != null) {
            //更新用户sessionKey
            userService.update(new LambdaUpdateWrapper<User>()
                    .set(User::getSessionKey, sessionKey)
                    .eq(User::getId, user.getId()));
            //判断账号是否停用
            if (Objects.equals(BanStatusEnum.BAN_STATUS.getStatus(), user.getStatus())) {
                return ResultVO.fail(HttpStatusEnum.ACCOUNT_HALT.getMsg());
            }
            //返回登录信息
            return ResultVO.ok(this.createUserLoginVO(user, userData));
        }
        //渠道标识
        Long agentId;
        Long fromUserId = null;
        if (param.getShareUserId() == null) {
            agentId = promoChannelService.getAgentIdByChannel(userData.getChannel());
        } else {
            UserDTO userDTO = userService.getUserByUserId(param.getShareUserId());
            agentId = userDTO.getAgentId();
            fromUserId = userDTO.getId();
        }
        //构造用户信息
        user = User.builder()
                .agentId(agentId)
                .gameId(userData.getGameId())
                .username(openId)
                .nickname(RegisterUtil.randomNickName(openId))
                .openId(openId)
                .sessionKey(sessionKey)
                .deviceType(userData.getDeviceType())
                .status(BanStatusEnum.NORMAL_STATUS.getStatus())
                .authentication(0)
                .createTime(LocalDateTime.now())
                .updateTime(LocalDateTime.now())
                .deviceSystem(userData.getDeviceSystem())
                .mac(userData.getMac())
                .imei(userData.getImei())
                .androidId(userData.getAndroidId())
                .ip(userData.getIp())
                .build();
        userService.save(user);
        gameUserService.createGameUser(user);
        callBackService.userCallBack(user);
        if (fromUserId != null) {
            UserShare userShare = UserShare.builder()
                    .gameId(userData.getGameId())
                    .fromUserId(fromUserId)
                    .toUserId(user.getId())
                    .createTime(LocalDateTime.now())
                    .updateTime(LocalDateTime.now())
                    .build();
            userShareService.save(userShare);
        }
        //返回登录信息
        return ResultVO.ok(this.createUserLoginVO(user, userData));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVO<UserLoginVO> loginPassword(LoginPasswordParam param, UserData userData) {
        //验证登录ip是否封禁
        if (ipBanService.checkIpBan(userData.getIp())) {
            return ResultVO.fail(HttpStatusEnum.IP_HALT.getMsg());
        }
        String username = param.getUsername();
        String password = param.getPassword();
        //用户信息
        User user;
        //登录, 进行登录检测
        if (Objects.equals(param.getType(), LoginPasswordParam.LOGIN)) {
            //判断是否手机号
            if (RegexUtil.checkPhone(username)) {
                user = userService.getOne(new LambdaQueryWrapper<User>()
                        .eq(User::getGameId, userData.getGameId()).eq(User::getMobile, username));
            } else {
                //根据用户名查询
                user = userService.getOne(new LambdaQueryWrapper<User>()
                        .eq(User::getGameId, userData.getGameId()).eq(User::getUsername, username));
            }
            //用户信息不存在
            if (user == null) {
                return ResultVO.fail(HttpStatusEnum.USERNAME_OR_PASSWORD_ERR.getMsg());
            }
            //判断账号是否停用
            if (Objects.equals(BanStatusEnum.BAN_STATUS.getStatus(), user.getStatus())) {
                return ResultVO.fail(HttpStatusEnum.ACCOUNT_HALT.getMsg());
            }
            //验证密码
            if (!Objects.equals(RegisterUtil.cmfPassword(password), user.getPassword())) {
                return ResultVO.fail(HttpStatusEnum.USERNAME_OR_PASSWORD_ERR.getMsg());
            }
        } else {
            //注册, 用户名密码校验
            HttpStatusEnum checkRegisterEnum = this.checkRegister(username, password);
            if (!Objects.equals(checkRegisterEnum, HttpStatusEnum.SUCCESS)) {
                return ResultVO.fail(checkRegisterEnum.getMsg());
            }
            //创建用户信息
            user = User.builder()
                    .agentId(promoChannelService.getAgentIdByChannel(userData.getChannel()))
                    .gameId(userData.getGameId())
                    .username(username)
                    .nickname(RegisterUtil.randomNickName(username))
                    .password(RegisterUtil.cmfPassword(password))
                    .deviceType(userData.getDeviceType())
                    .status(BanStatusEnum.NORMAL_STATUS.getStatus())
                    .authentication(0)
                    .createTime(LocalDateTime.now())
                    .updateTime(LocalDateTime.now())
                    .deviceSystem(userData.getDeviceSystem())
                    .mac(userData.getMac())
                    .imei(userData.getImei())
                    .androidId(userData.getAndroidId())
                    .ip(userData.getIp())
                    .build();
            userService.save(user);
            gameUserService.createGameUser(user);
            callBackService.userCallBack(user);
        }
        //返回登录信息
        return ResultVO.ok(this.createUserLoginVO(user, userData));

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVO<UserLoginVO> loginMobile(LoginMobileParam param, UserData userData) {
        //验证ip是否封禁
        if (ipBanService.checkIpBan(userData.getIp())) {
            return ResultVO.fail(HttpStatusEnum.IP_HALT.getMsg());
        }
        String mobile = param.getMobile();
        //验证码校验
        HttpStatusEnum httpStatusEnum = smsService.smsCheck(SmsTypeEnum.SMS_REG.getType(), mobile, param.getCode());
        //验证不通过, 返回
        if (!Objects.equals(httpStatusEnum, HttpStatusEnum.SUCCESS)) {
            return ResultVO.fail(httpStatusEnum.getMsg());
        }
        //获取用户信息
        User user = userService.getOne(new LambdaQueryWrapper<User>()
                .eq(User::getGameId, userData.getGameId()).eq(User::getMobile, mobile));
        //用户信息不存在, 直接注册
        if (user == null) {
            user = User.builder()
                    .agentId(promoChannelService.getAgentIdByChannel(userData.getChannel()))
                    .gameId(userData.getGameId())
                    .username(mobile)
                    .nickname(RegisterUtil.randomNickName(mobile))
                    .mobile(mobile)
                    .deviceType(userData.getDeviceType())
                    .status(BanStatusEnum.NORMAL_STATUS.getStatus())
                    .authentication(0)
                    .createTime(LocalDateTime.now())
                    .updateTime(LocalDateTime.now())
                    .deviceSystem(userData.getDeviceSystem())
                    .mac(userData.getMac())
                    .imei(userData.getImei())
                    .androidId(userData.getAndroidId())
                    .ip(userData.getIp())
                    .build();
            userService.save(user);
            gameUserService.createGameUser(user);
            callBackService.userCallBack(user);
        } else {
            //判断账号是否停用
            if (Objects.equals(BanStatusEnum.BAN_STATUS.getStatus(), user.getStatus())) {
                return ResultVO.fail(HttpStatusEnum.ACCOUNT_HALT.getMsg());
            }
        }
        //返回登录信息
        return ResultVO.ok(this.createUserLoginVO(user, userData));

    }

    private UserLoginVO createUserLoginVO(User user, UserData userData) {
        //获取token
        String userToken = userTokenService.getUserToken(user.getId(), userData.getDeviceType());
        //插入用户登录记录
        userLoginLogService.createUserLoginLog(user, LoginTypeEnum.LOGIN_IN.getLoginType());
        //查询用户实名信息
        UserCard userCard = userCardService.getOne(new LambdaQueryWrapper<UserCard>().eq(UserCard::getUserId, user.getId()));
        //查询游戏
        GameExt gameExt = gameExtService.getByGameId(userData.getGameId());
        //构造用户登录信息
        UserLoginVO userLoginVO = UserLoginVO.builder()
                .userId(user.getId())
                .userName(user.getUsername())
                .token(userToken)
                .authentication(user.getAuthentication())
                .bindPhone(Strings.isBlank(user.getMobile()) ? 0 : 1)
                .hasOldPassword(Strings.isNotBlank(user.getPassword()))
                .phoneNum(user.getShowPhoneNum())
                .cardName(userCard == null ? null : userCard.getShowCardName())
                .cardId(userCard == null ? null : userCard.getShowCardId())
                .checkSwitch(gameExt.getCheckSwitch())
                .build();
        log.error("用户登录返回登录信息, userLoginVO : {}", JsonUtil.toString(userLoginVO));
        return userLoginVO;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean loginOut(UserData userData) {
        User user = userService.getById(userData.getUserId());
        //新增用户退出记录
        return userLoginLogService.createUserLoginLog(user, LoginTypeEnum.LOGIN_OUT.getLoginType());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean writeOff(UserData userData) {
        //删除token
        userTokenService.deleteUserToken(userData.getUserId(), userData.getDeviceType());
        return Boolean.TRUE;
    }

    private HttpStatusEnum checkRegister(String username, String password) {
        //判断用户名是否存在敏感词
        if (wordCheckService.hasWord(username)) {
            return HttpStatusEnum.USERNAME_SENSITIVE;
        }
        //判断用户名是否已存在
        if (userService.count(new LambdaQueryWrapper<User>().eq(User::getUsername, username)) > 0) {
            return HttpStatusEnum.USERNAME_EXISTS;
        }
        //用户名合规检测
        HttpStatusEnum checkUserNameEnum = RegisterUtil.checkUserName(username);
        if (!Objects.equals(checkUserNameEnum, HttpStatusEnum.SUCCESS)) {
            return checkUserNameEnum;
        }
        //密码验证
        return RegisterUtil.checkPassword(password);
    }
}
